/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#include "StreamModel/service_xspf.h"
#include "utilities.h"
#include "debug.h"

#include <QFile>
#include <QXmlStreamReader>

/*
********************************************************************************
*                                                                              *
*    Class XspfStreams                                                         *
*                                                                              *
********************************************************************************
*/
XspfStreams::XspfStreams() : Service("Local file", SERVICE::LOCAL, false)
{
    /* restore state  */
    set_state(SERVICE::NO_DATA);
}

XspfStreams::XspfStreams(const QString & path) : Service("Local file", SERVICE::LOCAL, false)
{
    set_filename(path);
}

void XspfStreams::set_filename(const QString& path)
{
    m_filename = path;

    /* restore state  */
    set_state(SERVICE::NO_DATA);
}

void XspfStreams::setStreams(QList<MEDIA::TrackPtr> tracks)
{
    m_streams.clear();
    m_streams = tracks;
}

void XspfStreams::reload() 
{
    set_state(SERVICE::NO_DATA);  
    emit stateChanged();
}
    
void XspfStreams::load()
{
    m_streams.clear();

    Debug::debug() << "XspfStreams -> read file : " << m_filename;
    QFile file(m_filename);

    if(!file.exists()) {
      Debug::warning() << "XspfStreams -> file not find";
      set_state(SERVICE::DATA_OK);
      emit stateChanged();      
      return;
    }

    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
      Debug::warning() << "XspfStreams -> error opening file";
      set_state(SERVICE::NO_DATA);  
      emit stateChanged();      
      return;
    }

    QXmlStreamReader xml(&file);
    MEDIA::TrackPtr mi = MEDIA::TrackPtr(0);


    while(!xml.atEnd() && !xml.hasError())
    {
      xml.readNext();
      if (xml.isStartElement() && xml.name() == "title")
      {
        m_playlist_name = QString(xml.readElementText());
        //Debug::debug() << "-- XspfStreamList::read -> playlist_name : " << m_playlist_name;
      }
      else if (xml.isStartElement() && xml.name() == "trackList")
      {
        break;
      }
    }

    while(!xml.atEnd() && !xml.hasError())
    {
      xml.readNext();

      if (xml.isStartElement())
      {
          if(xml.name() == "track")
          {
            //! new stream entry
            //Debug::debug() << "-- XspfStreamList -> track start";
            mi = MEDIA::TrackPtr(new MEDIA::Track());
            mi->setType(TYPE_STREAM);
          }
          else if (xml.name() == "title" && mi)
          {
            mi->name = QString(xml.readElementText());
          }
          else if (xml.name() == "location" && mi)
          {
            mi->url  = QString(xml.readElementText());
          }
          else if (xml.name() == "category" && mi)
          {
            mi->categorie = QString(xml.readElementText());
          }
      }
      else if (xml.isEndElement() && xml.name() == "track")
      {
          //Debug::debug() << "-- XspfStreamList -> track end";
          //! if stream OK --> add it
          if (!mi->url.isEmpty() && !mi->name.isEmpty())
          {
            m_streams.append(mi);
          }
          mi = MEDIA::TrackPtr(0);
      }
    } // end while

    if(xml.hasError())
    {
      Debug::debug() << "XspfStreams -> error : " << xml.errorString();
    }

    qSort(m_streams.begin(), m_streams.end(), MEDIA::compareStreamCategorie);
    
    /* emit change */    
    set_state(SERVICE::DATA_OK);
    emit stateChanged();
}


void XspfStreams::save()
{
    //! open file to write
    QFile file(m_filename);

    if(file.exists()) file.remove();
    if(!file.open(QIODevice::WriteOnly)) return;

    QXmlStreamWriter xml(&file);
    xml.setAutoFormatting(true);

    xml.writeStartDocument();
    xml.writeStartElement("playlist");
    xml.writeAttribute("version", "1");
    xml.writeDefaultNamespace("http://xspf.org/ns/0/");

    xml.writeTextElement("title", m_playlist_name);
    //Debug::debug() << "-- XspfStreamList::write -> playlist_name : " << m_playlist_name;

    xml.writeStartElement("trackList");
    foreach (const MEDIA::TrackPtr mi, m_streams)
    {
        if (!mi) continue;

        xml.writeStartElement("track");
        xml.writeTextElement("location",  mi->url);
        xml.writeTextElement("title",     mi->name);
        xml.writeStartElement("extension");
        xml.writeAttribute("application", "yarock");
        xml.writeTextElement("category",  mi->categorie);
        xml.writeEndElement(); //extension
        xml.writeEndElement(); //track
    }
    xml.writeEndElement(); //trackList
    xml.writeEndElement(); //playlist
    xml.writeEndDocument();
}
